"""
Various helper functions for CLI commands.
"""

import sys
from pathlib import Path
from typing import Optional

import yaml
from rich.console import Console, Text

from . import aws, google
from .config import Config, ConfigNotFoundError, Environment, find_config

# Consoles for reporting errors and informational messages. Both write to stderr since the only
# thing which should be written to stdout is the console URL.
error_console = Console(markup=False, stderr=True, style="red")
info_console = Console(markup=False, stderr=True)


def set_quiet(quiet: bool):
    """
    Set quiet flag for info console.
    """
    info_console.quiet = quiet


def load_config(config_file: Optional[str | Path]) -> Config:
    """
    Load application configuration.

    Arguments:
        config_file: location of configuration file. If None, use the default search path

    Returns:
        the parsed configuration.
    """
    try:
        config_path = find_config(config_file)
    except ConfigNotFoundError:
        error_console.print(
            "Failed to locate a configuration file. Either provide the location explicitly",
            "with the",
            Text("--config-file", "yellow"),
            "option or ensure that a file called",
            Text(".aws-helper.yaml", "yellow"),
            "is present in the current directory or one of its parents.",
        )
        sys.exit(1)
    info_console.print(
        "Loading configuation from:", Text(str(config_path), "yellow"), style="green"
    )

    try:
        with config_path.open() as f:
            return Config.model_validate(yaml.safe_load(f))
    except Exception as e:
        error_console.print(
            "Error loading configuration from ",
            Text(f'"{config_path.as_posix()}"', "yellow"),
            f": {e}",
            sep="",
        )
        sys.exit(1)


def get_environment(config: Config, environment: str) -> Environment:
    """
    Get configuration for a named environment from the global config.
    """
    try:
        return config.environments[environment]
    except KeyError:
        error_console.print(
            'Failed to find configuration for environment "',
            Text(environment, "yellow"),
            '". Available environments: ',
            Text(", ".join(config.environments.keys()), "yellow"),
            sep="",
        )
        sys.exit(1)


def google_id_token(environment_config: Environment) -> tuple[str, google.UserInfo]:
    """
    Get an appropriate Google OpenID id token for the given envirionment.

    Arguments:
        environment_config: environment-specific configuration loaded from the config file

    Returns:
        The id token and a UserInfo object describing the authenticated user.
    """
    try:
        google_creds = google.get_default_credentials(
            environment_config.google_quota_project_id,
        )
        user_info = google.get_user_info(google_creds)
    except Exception as e:
        error_console.print(f"Failed to get Google access token. Error was: {e}")
        error_console.print(
            "Hint: ensure that you have an up-to-date sign in token for an Google Cloud admin",
            "account via",
            Text("gcloud auth application-default login", "blue"),
            style="yellow",
        )
        sys.exit(1)

    info_console.print(
        "Authenticated with Google account ",
        Text(user_info.email, "yellow"),
        ".",
        sep="",
        style="green",
    )

    try:
        id_token = google.get_id_token(
            google_creds,
            environment_config.google_service_account_email,
            environment_config.aws_role_arn,
        )
    except Exception as e:
        error_console.print(
            "Failed to create id token for",
            Text(environment_config.google_service_account_email, "yellow"),
            f"Error was: {e}",
        )
        error_console.print(
            "Hint: ensure that you have an up-to-date sign in token for an Google Cloud admin",
            "account via",
            Text("gcloud auth application-default login", "blue"),
            "and that",
            Text(user_info.email, "yellow"),
            "has rights to impersonate the AWS Console service",
            "account.",
            style="yellow",
        )
        sys.exit(1)

    return id_token, user_info


def aws_credentials(
    google_id_token: str,
    role_arn: str,
    role_session_name: str,
    profile_name: Optional[str],
) -> aws.Credentials:
    """
    Get temporary AWS credentials given a Google id token.
    """
    try:
        return aws.assume_role_with_web_identity(
            google_id_token,
            role_arn=role_arn,
            role_session_name=role_session_name,
            profile_name=profile_name,
        )
    except Exception as e:
        error_console.print(f"Failed to obtain AWS credentials. Error was: {e}")
        sys.exit(1)
